//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include "psocdynamic.h"

#define MMC_SCS 0x01	//p[1]0
#define SW1 0x01

unsigned char PWM_End_Flag;
unsigned char pInData;
unsigned char RecPlay;			//FLAG for changing Rec/Play mode(0:Play encoded file, 1:Rec mode, 2:Play recorded file)

#pragma fastcall16 SPI_SEND
#pragma fastcall16 ADPCM_INIT
#pragma fastcall16 ADPCMDecoder1
#pragma fastcall16 ADPCMDecoder2

extern unsigned char SPI_SEND(unsigned char UUBUF);		//read from assemble file
extern void ADPCM_INIT(void);
extern unsigned char ADPCMDecoder1(unsigned char code);
extern unsigned char ADPCMDecoder2(unsigned char code);


/*
unsigned char SPI_SEND(unsigned char UUBUF){
	SPIM_1_SendTxData(UUBUF);
	while(!(SPIM_1_bReadStatus() & SPIM_1_SPIM_SPI_COMPLETE)); //transmit complete?
    return(SPIM_1_bReadRxData());
}
*/
unsigned char MMC_CMD(unsigned char CMD,unsigned long ADDR,unsigned char MMC_CRC){
	unsigned char USERBUF;
	SPI_SEND(0xff);
	SPI_SEND(0x40 + CMD);						
	SPI_SEND((ADDR >> 24) & 0xff);
	SPI_SEND((ADDR >> 16) & 0xff);
	SPI_SEND((ADDR >> 8) & 0xfe);
	SPI_SEND(/*ADDR  & */0x00);
	SPI_SEND(MMC_CRC & 0xFF);		
	//----------------------------
	//	wait R1 resp
	//----------------------------
	for(;;){
		USERBUF	=	SPI_SEND(0xff);
		if(!(USERBUF & 0x80))break;	
	}//timed out
	return(USERBUF);
}

void MMC_INIT(void){
	unsigned char LP1;
	// ----------- SD memory Card initialize ------------
	// 74 or more cycle dummy clock sending
	// --------------------------------------------------
	pInData = PRT1DR;
	pInData |= MMC_SCS;
	PRT1DR = pInData;		// CS = High
	for(LP1=100;LP1!=0;LP1--)SPI_SEND(0xff);
	// -------- dummy clock send was over ---------------
	// ------- Send CMD0 to MMC or SD memory Card -------
	pInData = PRT1DR;
	pInData &= ~MMC_SCS;
	PRT1DR = pInData;		// CS = Low
	MMC_CMD(0,0,0x95);		//CMD0,CRC=0x95
	// -------- Send ACMD41 to SD memory Card -------
	// -------- 	ACMD41 = CMD55 + CMD41	  -------
	do{
		MMC_CMD(55,0,0xFF);				//CMD55,CRC=0xFFh
	}while(MMC_CMD(41,0,0x01) != 0x00);	//CMD41,CRC=0x01h 
	// --------		ACMD41 send was End			-------
	// ------------------------------------------------
	while(!SPI_SEND(0xff));	//wait busy
	while(!SPI_SEND(0xff));	//wait busy
	MMC_CMD(16,512,0x01);	//SET_BLOCKLEN = 512 byte
	
}

void MMC_SEND_STOP(void){
	unsigned char LP1;
	
	pInData = PRT1DR;
//	pInData |= MMC_SCS;
//	PRT1DR = pInData;		// CS = High
	pInData &= ~MMC_SCS;
	PRT1DR = pInData;		// CS = Low	
	for(LP1=100;LP1!=0;LP1--)SPI_SEND(0xff);
	MMC_CMD(12,0,0x01);		//CMD12
	while(!SPI_SEND(0xff));	//wait busy
	while(!SPI_SEND(0xff));	//wait busy
	for(LP1=100;LP1!=0;LP1--)SPI_SEND(0xff);
}

// Registors To read FAT
// Supported FAT16 only
unsigned char Sector_Per_Cluster;

unsigned long SectorZero;
unsigned long Beginning_Of_FAT_Sector;
unsigned long Reserved_for_FAT_Sector;
unsigned long Beginning_Of_Dir_Sector;
unsigned long Beginning_Of_Data_Sector;
unsigned long Now_Cluster;

void Get_Sector_Zero(void){
	unsigned int  LP1;
	SectorZero = 0;
	
	do{
		for(LP1=100;LP1!=0;LP1--)SPI_SEND(0xff);					
	}while(MMC_CMD(18,0x00,0x01)!= 0x00);	//end do
	//CMD18=52_(4-byte data address)_01h	
	//-----------------------------
	// read start
	//-----------------------------		
	while(SPI_SEND(0xff) != 0xfe);	//get start token?
	for(LP1=0;LP1<512;LP1++){
		switch(LP1){
			case 454:
				SectorZero = SPI_SEND(0xff);
				break;
			default:
				SPI_SEND(0xff);
				break;
		}// end switch(LP1)
	}	//end for(LP1=0;LP1<512;LP1++)
	SPI_SEND(0xff);		//16-bit CRC
	SPI_SEND(0xff);		//16-bit CRC
	MMC_SEND_STOP();
	
}


void Read_BPB(void){
	unsigned int LP1;
	
	do{
		for(LP1=100;LP1!=0;LP1--)SPI_SEND(0xff);					
	}while(MMC_CMD(18,(SectorZero << 9),0x01)!= 0x00);	//end do
	//CMD18=52_(4-byte data address)_01h	
	//-----------------------------
	// read start
	//-----------------------------		
	while(SPI_SEND(0xff) != 0xfe);	//get start token?
	for(LP1=0;LP1<512;LP1++){
		switch(LP1){
			case 13:
				Sector_Per_Cluster = SPI_SEND(0xff);
				break;
			case 14:
				Beginning_Of_FAT_Sector = SPI_SEND(0xff);
				break;
			case 22:
				Reserved_for_FAT_Sector = SPI_SEND(0xff); 
				break;
			default:
				SPI_SEND(0xff);
				break;
		}
		
	}	//end for(LP1=0;LP1<512;LP1++)
	SPI_SEND(0xff);		//16-bit CRC
	SPI_SEND(0xff);		//16-bit CRC
	MMC_SEND_STOP();
	
	Beginning_Of_Dir_Sector = Beginning_Of_FAT_Sector + (Reserved_for_FAT_Sector * 2);
	Beginning_Of_Data_Sector = Beginning_Of_Dir_Sector + 32;
}

unsigned long Get_File_Num(unsigned int Num){
	unsigned long File_Length_Sector = 0;
	unsigned int LP1;
	unsigned char LP2;

	unsigned char UUBUF[32];
	if(Num > 16)Num = 16;
	
	do{
		for(LP1=100;LP1!=0;LP1--)SPI_SEND(0xff);					
	}while(MMC_CMD(18,((Beginning_Of_Dir_Sector + SectorZero) << 9),0x01)!= 0x00);	//end do
	//CMD18=52_(4-byte data address)_01h	
	//-----------------------------
	// read start
	//-----------------------------		
	while(SPI_SEND(0xff) != 0xfe);	//get start token?
	for(LP1=0;LP1 < Num;LP1++){
		for(LP2=0;LP2<32;LP2++){
			UUBUF[LP2] = SPI_SEND(0xff);
		}
		if((UUBUF[11]!=0x20)||(UUBUF[0]==0xE5)||(UUBUF[26]==0x00)){//deleted file or folder?
			Num++;
			if(Num > 16)Num = 1;
		}
	}	//end for(LP1=0;LP1<Num;LP1++)
	Now_Cluster = (UUBUF[27]<< 8)|UUBUF[26];
	
	File_Length_Sector = (UUBUF[31]<< 8)|UUBUF[30];
	File_Length_Sector <<= 8;	//bug of C compiler
	File_Length_Sector |= UUBUF[29];
	
	//File_Length_Sector= *(unsigned long *)&UUBUF[31];
	
	File_Length_Sector >>= 1;	//512byte order
	File_Length_Sector &= 0x007FFFFF;
	MMC_SEND_STOP();
	return(File_Length_Sector);
}



void main(void){

	unsigned char UUBUF;
	unsigned char TrackNum;
	unsigned char LP1;
	unsigned long FileLength,LP2;
	RecPlay = 0;
	LoadConfig_miniadpcm27143();	//loading base config
	SPIM_1_Start(SPIM_1_SPIM_MODE_0 | SPIM_1_SPIM_MSB_FIRST);
	PWM8_3_Start();
	
	
_PLAY_MODE:

	if(PRT1DR & SW1)RecPlay = 1;
	else RecPlay = 0;
	while(PRT1DR & SW1);	//wait until release SW1
	MMC_INIT();
	//init to read FAT
	Get_Sector_Zero();
	Read_BPB();
	TrackNum = 2;
	if((TrackNum == 1)&&(RecPlay != 1))RecPlay = 2;
	FileLength = Get_File_Num(TrackNum);
	
	LoadConfig_PlayConfig();		//loading play mode config
	
	PWM8_1_Start();	
	PWM8_2_Start();
	ADPCM_INIT();
	PWM_End_Flag = 1;	// 0:Go, 1:Stop
	PRT1DR = pInData;
	M8C_EnableGInt;
	M8C_EnableIntMask(INT_MSK1,INT_MSK1_DCB03);	
	if(RecPlay == 1)goto _REC_MODE;
	
	while(1){
		do{
			for(LP1=100;LP1!=0;LP1--)SPI_SEND(0xff);					
		}while(MMC_CMD(18,((((Now_Cluster - 2) * Sector_Per_Cluster)+Beginning_Of_Data_Sector + SectorZero)<<9),0x01)!= 0x00);	//end do
		//CMD18=52_(4-byte data address)_01h	
		//-----------------------------
		// read start
		//-----------------------------
		if(RecPlay == 2)goto _PLAY_WAV;
		while(--FileLength){
		
			while(SPI_SEND(0xff) != 0xfe);	//get start token?
			LP1=128;
			while(LP1--){
				UUBUF	=	SPI_SEND(0xff);
				PWM8_1_WritePulseWidth(ADPCMDecoder1(UUBUF));			
				PWM8_2_WritePulseWidth(ADPCMDecoder2(UUBUF >> 4));
				while(PWM_End_Flag);
				PWM_End_Flag = 1;
				UUBUF	=	SPI_SEND(0xff);
				PWM8_1_WritePulseWidth(ADPCMDecoder1(UUBUF));			
				PWM8_2_WritePulseWidth(ADPCMDecoder2(UUBUF >> 4));
				while(PWM_End_Flag);
				PWM_End_Flag = 1;
				UUBUF	=	SPI_SEND(0xff);
				PWM8_1_WritePulseWidth(ADPCMDecoder1(UUBUF));			
				PWM8_2_WritePulseWidth(ADPCMDecoder2(UUBUF >> 4));
				while(PWM_End_Flag);
				PWM_End_Flag = 1;
				UUBUF	=	SPI_SEND(0xff);
				PWM8_1_WritePulseWidth(ADPCMDecoder1(UUBUF));			
				PWM8_2_WritePulseWidth(ADPCMDecoder2(UUBUF >> 4));
				while(PWM_End_Flag);
				PWM_End_Flag = 1;
			}	//end while(LP1--)
			SPI_SEND(0xff);		//16-bit CRC
			SPI_SEND(0xff);		//16-bit CRC
			if(PRT1DR & SW1)break;
	
		}	//end while(--FileLength)
_STOP_TRAN:
////////////////////////////////////////
/////////		STOP Tran 
////////////////////////////////////////
	for(LP2=16000;LP2!=0;LP2--){
		while(PWM_End_Flag);
		PWM_End_Flag = 1;
		if(!(PRT1DR & SW1))break;
	}
	
	if(LP2==0)RecPlay = 2;
	else RecPlay = 0;// long time push
	while(PRT1DR & SW1);
	MMC_SEND_STOP();
	ADPCM_INIT();
	TrackNum++;
	if(!Get_File_Num(TrackNum))TrackNum = 1;
	if(RecPlay==2)TrackNum = 1;
	if((RecPlay==0)&&(TrackNum==1))TrackNum = 2;
	FileLength = Get_File_Num(TrackNum);
	}//end while(1)	
	
	
//play wave file(12kHz mono)
_PLAY_WAV:
	while(--FileLength){
	
		while(SPI_SEND(0xff) != 0xfe);	//get start token?
		LP1=128;
		while(LP1--){
			PWM8_1_WritePulseWidth(SPI_SEND(0xff));			
			//PWM8_2_WritePulseWidth(SPI_SEND(0xff));
			while(PWM_End_Flag);
			PWM_End_Flag = 1;
			PWM8_1_WritePulseWidth(SPI_SEND(0xff));			
			//PWM8_2_WritePulseWidth(SPI_SEND(0xff));
			while(PWM_End_Flag);
			PWM_End_Flag = 1;
			PWM8_1_WritePulseWidth(SPI_SEND(0xff));			
			//PWM8_2_WritePulseWidth(SPI_SEND(0xff));
			while(PWM_End_Flag);
			PWM_End_Flag = 1;
			PWM8_1_WritePulseWidth(SPI_SEND(0xff));			
			//PWM8_2_WritePulseWidth(SPI_SEND(0xff));
			while(PWM_End_Flag);
			PWM_End_Flag = 1;
		}	//end while(LP1--)
		SPI_SEND(0xff);		//16-bit CRC
		SPI_SEND(0xff);		//16-bit CRC
		if(PRT1DR & SW1)break;

	}	//end while(--FileLength)
	goto _STOP_TRAN;














_REC_MODE:
	UnloadConfig_PlayConfig();
	LoadConfig_RecConfig();
	PGA_1_Start(PGA_1_MEDPOWER);
	DELSIG8_1_Start( DELSIG8_1_HIGHPOWER );
	DELSIG8_1_StartAD(); 
	TrackNum = 1;
	FileLength = Get_File_Num(TrackNum);

		do{
			for(LP1=100;LP1!=0;LP1--)SPI_SEND(0xff);					
		}while(MMC_CMD(25,((((Now_Cluster - 2) * Sector_Per_Cluster)+Beginning_Of_Data_Sector + SectorZero)<<9),0x01)!= 0x00);	//end do
		//CMD25
		while(!SPI_SEND(0xff));
		while(!SPI_SEND(0xff));
		//-----------------------------
		// write start
		//-----------------------------		
		while(--FileLength){
			while(!SPI_SEND(0xff));	//wait busy
			SPI_SEND(0xfc);			//send start token
			LP1=128;
			while(LP1--){
				while(!(DELSIG8_1_fIsDataAvailable()));
				SPI_SEND(DELSIG8_1_cGetDataClearFlag());
				while(PWM_End_Flag);
				PWM_End_Flag = 1;
				while(!(DELSIG8_1_fIsDataAvailable()));
				SPI_SEND(DELSIG8_1_cGetDataClearFlag());
				while(PWM_End_Flag);
				PWM_End_Flag = 1;
				while(!(DELSIG8_1_fIsDataAvailable()));
				SPI_SEND(DELSIG8_1_cGetDataClearFlag());
				while(PWM_End_Flag);
				PWM_End_Flag = 1;
				while(!(DELSIG8_1_fIsDataAvailable()));
				SPI_SEND(DELSIG8_1_cGetDataClearFlag());
				while(PWM_End_Flag);
				PWM_End_Flag = 1;
			}	//end while(LP1--)
			SPI_SEND(0xff);		//16-bit CRC
			SPI_SEND(0xff);		//16-bit CRC
			if(PRT1DR & SW1)break;
			SPI_SEND(0xff);	//get resp(dummy read)
		}	//end while(--FileLength)
		

////////////////////////////////////////
/////////		STOP Tran 
////////////////////////////////////////
	while(PRT1DR & SW1);
	MMC_SEND_STOP();
//	ADPCM_INIT();
//	TrackNum++;
//	if(!Get_File_Num(TrackNum))TrackNum = 1;
//	FileLength = Get_File_Num(TrackNum);
//	while(!(PRT1DR & SW1));
//	while(PRT1DR & SW1);
	RecPlay = 0;
	UnloadConfig_RecConfig();
	goto _PLAY_MODE;
	
}
/*
#pragma interrupt_handler PWM8_3_INT 
void PWM8_3_INT(void)
{
	PWM_End_Flag = 0;//0:Go
}
*/